
#!/usr/bin/python

# Import von Bibliotheken
import time
import RPi.GPIO as GPIO

# Pinbelegung
LCD_RS = 22      # Register Select Pin
LCD_EN  = 11      # Enable Pin
LCD_DATA4 = 23   # Data Pins
LCD_DATA5 = 10
LCD_DATA6 = 9
LCD_DATA7 = 25

LCD_WIDTH = 16 		# Spaltenanzahl
LCD_LINE_1 = 0x80 	# erste Zeile
LCD_LINE_2 = 0xC0 	# zweite Zeile
LCD_CHR = GPIO.HIGH  # Constraint Handling Rules
LCD_CMD = GPIO.LOW  # Command
EN_PULSE = 0.0005  # Impuls
EN_PAUSE = 0.0005  # Pause

# Impulse fuer den Enable Pin
def enable(bits, mode):
	# Pins auf LOW setzen
	GPIO.output(LCD_RS, mode)
	GPIO.output(LCD_DATA4, GPIO.LOW)
	GPIO.output(LCD_DATA5, GPIO.LOW)
	GPIO.output(LCD_DATA6, GPIO.LOW)
	GPIO.output(LCD_DATA7, GPIO.LOW)
 
	if bits & 0x10 == 0x10:
	  GPIO.output(LCD_DATA4, GPIO.HIGH)
	if bits & 0x20 == 0x20:
	  GPIO.output(LCD_DATA5, GPIO.HIGH)
	if bits & 0x40 == 0x40:
	  GPIO.output(LCD_DATA6, GPIO.HIGH)
	if bits & 0x80 == 0x80:
	  GPIO.output(LCD_DATA7, GPIO.HIGH)
     
	time.sleep(EN_PAUSE)    
	GPIO.output(LCD_EN, GPIO.HIGH)  
	time.sleep(EN_PULSE)
	GPIO.output(LCD_EN, GPIO.LOW)  
	time.sleep(EN_PAUSE)      
	GPIO.output(LCD_DATA4, GPIO.LOW)
	GPIO.output(LCD_DATA5, GPIO.LOW)
	GPIO.output(LCD_DATA6, GPIO.LOW)
	GPIO.output(LCD_DATA7, GPIO.LOW)
 
	if bits&0x01==0x01:
	  GPIO.output(LCD_DATA4, GPIO.HIGH)
	if bits&0x02==0x02:
	  GPIO.output(LCD_DATA5, GPIO.HIGH)
	if bits&0x04==0x04:
	  GPIO.output(LCD_DATA6, GPIO.HIGH)
	if bits&0x08==0x08:
	  GPIO.output(LCD_DATA7, GPIO.HIGH)
     
	time.sleep(EN_PAUSE)    
	GPIO.output(LCD_EN, GPIO.HIGH)  
	time.sleep(EN_PULSE)
	GPIO.output(LCD_EN, GPIO.LOW)  
	time.sleep(EN_PAUSE)  

# Initialisierung
def display_init():
	enable(0x33, LCD_CMD)
	enable(0x32, LCD_CMD)
	enable(0x28, LCD_CMD)
	enable(0x0C, LCD_CMD)  
	enable(0x06, LCD_CMD)
	enable(0x01, LCD_CMD)  

# Anzeigen der Nachrichten
def lcd_message(message):
	message = message.ljust(LCD_WIDTH," ")  
	for i in range(LCD_WIDTH):
	  enable(ord(message[i]),LCD_CHR)
	
# Hauptprogramm
if __name__ == '__main__':
	GPIO.setmode(GPIO.BCM)
	GPIO.setwarnings(False)
	GPIO.setup(LCD_EN, GPIO.OUT)
	GPIO.setup(LCD_RS, GPIO.OUT)
	GPIO.setup(LCD_DATA4, GPIO.OUT)
	GPIO.setup(LCD_DATA5, GPIO.OUT)
	GPIO.setup(LCD_DATA6, GPIO.OUT)
	GPIO.setup(LCD_DATA7, GPIO.OUT)

	display_init()

	# erste Anzeige
	enable(LCD_LINE_1, LCD_CMD)
	lcd_message("Dieser LCD")
	enable(LCD_LINE_2, LCD_CMD)
	lcd_message("funktioniert! :)")
	
	time.sleep(4)
	
	# anzuzeigende Nachrichten
	nachricht1 = "Dies ist ein"
	nachricht2 = "Testdurchlauf."
	nachricht3 = "xxxx Joy-IT xxxx"
	nachricht4 = "x Explorer 500 x"
 
	# erster Nachrichtenblock
	for i in range(len(nachricht1)):
		enable(LCD_LINE_1, LCD_CMD)
		lcd_message(nachricht1[:i+1])
		enable(LCD_LINE_2, LCD_CMD)
		lcd_message("")
		time.sleep(0.1)	
   
	for i in range(len(nachricht2)):
		enable(LCD_LINE_1, LCD_CMD)
		lcd_message(nachricht1)
		enable(LCD_LINE_2, LCD_CMD)
		lcd_message(nachricht2[:i+1])
		time.sleep(0.1)
  
  # Pause fuer 5 Sekunden
	time.sleep(5)
 
	# zweiter Nachrichtenblock
	for i in range(len(nachricht3)):
		enable(LCD_LINE_1, LCD_CMD)
		lcd_message(nachricht3[:i+1])
		enable(LCD_LINE_2, LCD_CMD)
		lcd_message("")
		time.sleep(0.1)	
   
	for i in range(len(nachricht4)):
		enable(LCD_LINE_1, LCD_CMD)
		lcd_message(nachricht3)
		enable(LCD_LINE_2, LCD_CMD)
		lcd_message(nachricht4[:i+1])
		time.sleep(0.1)
   
	time.sleep(4)
	# Beenden des Programmes
	GPIO.cleanup()